@@ -1,18 +1,19 @@
-"""
-This developer kit is meant for advanced users of the `Scribble Enrich Feature Store`. 
-This includes command line scripts to create and manage Enrich modules and APIs to 
-write new modules on the platform::
-  
-      ______           _      __       _____ ____  __ __
-     / ____/___  _____(_)____/ /_     / ___// __ \\/ //_/
-    / __/ / __ \\/ ___/ / ___/ __ \\    \\__ \\/ / / / ,<   
-   / /___/ / / / /  / / /__/ / / /   ___/ / /_/ / /| |  
-  /_____/_/ /_/_/  /_/\\___/_/ /_/   /____/_____/_/ |_|  
-  
+r"""
+This developer kit is meant for advanced users of the `Enrich Full-Stack Feature Store`. This includes command line scripts to create and manage Enrich modules and APIs to write new modules on the platform:
+
+<pre>
+ _____            _      _       ____  ____  _  __
+| ____|_ __  _ __(_) ___| |__   / ___||  _ \| |/ /
+|  _| | '_ \| '__| |/ __| '_ \  \___ \| | | | ' /
+| |___| | | | |  | | (__| | | |  ___) | |_| | . \
+|_____|_| |_|_|  |_|\___|_| |_| |____/|____/|_|\_\
+
+</pre>
+
 """
 import os
 
-VERSION = "4.3.8"
+VERSION = "4.6.0"
 
 
 def _get_git_revision(path):
@@ -23,10 +24,10 @@ def _get_git_revision(path):
 
 
 def get_revision():
-    """
-    :returns: Revision number of this branch/checkout, if available. None if
-        no revision number can be determined.
-    """
+    #
+    #:returns: Revision number of this branch/checkout, if available. None if
+    #    no revision number can be determined.
+    #
     package_dir = os.path.dirname(__file__)
     checkout_dir = os.path.normpath(os.path.join(package_dir, os.pardir))
     path = os.path.join(checkout_dir, ".git")
@@ -35,6 +36,11 @@ def get_revision():
 
 
 def get_version():
+    """
+    Revision number of this branch/checkout, if available. None if
+    no revision number can be determined.
+    """
+
     base = VERSION
     if __build__:
         base = "%s (%s)" % (base, __build__)
