@@ -183,7 +183,7 @@ class DBTLineageMetadata(MetadataBase):
 
         message = event["message"]
 
-        if not ((re.search("On \S+:", message)) and ('"app": "dbt"' in message)):
+        if not ((re.search(r"On \S+:", message)) and (r'"app": "dbt"' in message)):
             return {}
 
         lines = message.split("\n")
@@ -192,7 +192,7 @@ class DBTLineageMetadata(MetadataBase):
         sql = ""
         join = False
         for idx, m in enumerate(lines):
-            if (re.match("^On \S+:", m)) and ('"app": "dbt"' in m):
+            if (re.match(r"^On \S+:", m)) and ('"app": "dbt"' in m):
                 join = True
                 continue
             if join:
