@@ -7,7 +7,8 @@ __all__ = [
     "MetricsBase",
     "AnomaliesBase",
     "SyntheticDataGeneratorBase",
-    "DataObserverBase"
+    "DataObserverBase",
+    "ChangePointDetectorBase"
 ]
 
 from .fileops import *
@@ -17,6 +18,7 @@ from .notebook_executor import *
 from .feature_compute import *
 from .metrics import *
 from .anomalies import *
+from .changepoints import *
 from .synthetic_data_generator import *
 from .observability import *
 from .data_quality import *
