@@ -12,6 +12,8 @@ import seaborn as sns
 import matplotlib.pyplot as plt
 import logging
 
+from pandas.api.types import is_numeric_dtype
+
 from enrichsdk.contrib.lib.transforms import note
 from enrichsdk.contrib.lib.assets import profilespec
 from enrichsdk.datasets import TransformDoodle
@@ -113,20 +115,29 @@ class AnomaliesBase(Compute):
         handlers = self.get_handlers(spec)
 
         # use the specified strategy to detect anomalies
-        strategy = config.get("strategy")
-        if strategy == None:
-            # We are doing the default anomaly detection which is deviation from mean
-            data = self.process_spec_default(data, spec)
-        elif strategy == "custom":
-            # Custom callback
-            data  = self.process_spec_custom(data, spec)
-        else:
-            # for now, we fallback to the default strategy
-            data  = self.process_spec_default(data, spec)
+        try:
+            strategy = config.get("strategy")
+            if strategy == None:
+                # We are doing the default anomaly detection which is deviation from mean
+                data = self.process_spec_default(data, spec)
+            elif strategy == "custom":
+                # Custom callback
+                data  = self.process_spec_custom(data, spec)
+            else:
+                # for now, we fallback to the default strategy
+                data  = self.process_spec_default(data, spec)
 
-        # The spec processor can return multiple dataframes
-        if isinstance(data, pd.DataFrame):
-            data = {name: data, "stats": {}}
+            # The spec processor can return multiple dataframes
+            if isinstance(data, pd.DataFrame):
+                data = {name: data, "stats": {}}
+        except:
+            logger.exception(f"Failed to process {name}",
+                             extra={
+                                 'transform': self.name,
+                                 'data': msg
+                             })
+
+            return None
 
         logger.debug(
             f"Processed anomalies",
@@ -234,6 +245,10 @@ class AnomaliesBase(Compute):
             # make a copy of the df, we'll keep adding anomlay metrics to it
             df = orig_df[columns].copy()
 
+            if not is_numeric_dtype(df[metric]):
+                msg += f"{metric} Metric not numeric. Skipping\n"
+                continue
+
             # compute the anomalies for this metric
             points      = df[metric].to_numpy()     # all data as an MxN matrix
             centroid    = df[metric].mean()          # the computed centroid of the dataset
@@ -305,6 +320,10 @@ class AnomaliesBase(Compute):
             if group_name == None or metrics == None:
                 continue
 
+            if not all([is_numeric_dtype(df[metric]) for metric in g_metrics]):
+                msg += f"{group_name} One or more metrics are not numeric\n"
+                continue
+
             # make a copy of the df, we'll keep adding anomlay metrics to it
             df = orig_df[columns].copy()
 
@@ -366,6 +385,14 @@ class AnomaliesBase(Compute):
                              'data': msg
                          })
 
+        if len(dfs) == 0:
+            logger.debug(f"{name}: No outputs computed",
+                         extra={
+                             'transform': self.name,
+                             'data': msg
+                         })
+            return None
+
         #########
         # construct the DF for output
         #########
@@ -838,6 +865,7 @@ class AnomaliesBase(Compute):
 
         # Now go through each spec and generate anomaly reports
         for spec in profile:
+
             ## first, some checks on the spec
             do_process_spec = True
             name = spec.get('name', 'NO_SPEC_NAME')
@@ -872,26 +900,37 @@ class AnomaliesBase(Compute):
                     )
                     do_process_spec = False
                     break
-            if do_process_spec == False:
+
+            if not do_process_spec:
                 continue
 
             ## pre-process the spec
-            spec = self.preprocess_spec(spec)
-            logger.debug(f"Preproccessed spec: {spec['name']}",
+            try:
+                spec = self.preprocess_spec(spec)
+                logger.debug(f"Preproccessed spec: {spec['name']}",
                              extra={
                                  'transform': self.name,
                                  'data': json.dumps(spec, indent=4)
                              })
 
-            ## we can now proceed with processing the spec
-            # frist, load the source data
-            data = self.load_dataset(spec)
+                ## we can now proceed with processing the spec
+                # frist, load the source data
+                data = self.load_dataset(spec)
 
-            ## process the spec to detect outliers
-            data = self.process_spec(spec, data)
+                ## process the spec to detect outliers
+                data = self.process_spec(spec, data)
 
-            # write the detected outliers
-            self.store_result(spec, data)
+                if ((not isinstance(data, dict)) or
+                    (len(data) == 0)):
+                    continue
+
+                # write the detected outliers
+                self.store_result(spec, data)
+            except:
+                logger.exception(f"Failed to process {name}",
+                                 extra={
+                                     'transform': self.name
+                                 })
 
         # Done
         logger.debug(
