@@ -11,34 +11,22 @@ import traceback
 import subprocess
 import numpy as np
 import pandas as pd
-from enrichsdk import Compute, S3Mixin
+from enrichsdk import S3Mixin, Compute
+from enrichsdk.contrib.lib.assets import AnonymizerMixin
+
 from datetime import datetime, date, timedelta
 from dateutil import parser as dateparser, relativedelta
 import logging
 
 logger = logging.getLogger("app")
 
-from enrichsdk.utils import get_lineage_of_query
+from enrichsdk.utils import (get_lineage_of_query,
+                             note,
+                             SafeEncoder,
+                             get_yesterday,
+                             get_today)
 
-
-class CustomEncoder(json.JSONEncoder):
-    def default(self, obj):
-        try:
-            return super().default(obj)
-        except:
-            return str(obj)
-
-
-def get_yesterday():
-    yesterday = date.today() + timedelta(days=-1)
-    return yesterday.isoformat()
-
-
-def get_today():
-    return date.today().isoformat()
-
-
-class InMemoryQueryExecutorBase(Compute):
+class InMemoryQueryExecutorBase(AnonymizerMixin, Compute):
     """
     Base class for an InMemory QueryExecutor transform. This is useful
     to run queries against backends such as backends such as
@@ -134,7 +122,9 @@ class InMemoryQueryExecutorBase(Compute):
         for s in specs:
             if not s.get("enable", True):
                 continue
-            categories = s.get("categories", [])
+            categories = s.get("categories", s.get('category', []))
+            if isinstance(categories, str):
+                categories = [categories]
             for c in categories:
                 if c not in targets:
                     targets.append(c)
@@ -186,16 +176,22 @@ class InMemoryQueryExecutorBase(Compute):
         database = engine.url.database
 
         # Insert extra dependencies
-        dependencies += get_lineage_of_query(engine, sql)
-
-        lineage = {"type": "lineage", "dependencies": dependencies}
+        try:
+            dependencies += get_lineage_of_query(engine, sql)
+        except:
+            dependencies = []
+            logger.warning("Unable to get lineage",
+                             extra={
+                                 'transform': self.name,
+                                 'data': f"SQL being checked:\n {sql}"
+                             })
 
         # Generate column information...
         columns = self.get_column_metadata(name, df)
 
         ## => Gather the update parameters
         updated_detail = {
-            "df": None,
+            "df": df,
             "description": f"Output for query {name}",
             "transform": self.name,
             "frametype": "pandas",
@@ -204,10 +200,13 @@ class InMemoryQueryExecutorBase(Compute):
                     "type": "compute",
                     "columns": columns,
                 },
-                lineage,
             ],
         }
 
+        if len(dependencies) > 0:
+            lineage = {"type": "lineage", "dependencies": dependencies}
+            updated_detail['params'].append(lineage)
+
         # Dump it into the shared state
         self.state.update_frame(name, updated_detail, create=True)
 
@@ -418,6 +417,10 @@ class InMemoryQueryExecutorBase(Compute):
         # Get the registry
         self.registry = self.get_registry()
 
+        # => Initialize anonymization if required
+        if 'anonymization' in self.args:
+            self.anonymize_init(self.args['anonymization'])
+
         # List of specification names
         targets = self.args["targets"]
 
@@ -430,7 +433,10 @@ class InMemoryQueryExecutorBase(Compute):
             try:
 
                 name = spec["name"]
-                categories = spec.get("categories", ["all"])
+                categories = spec.get("categories",
+                                      spec.get('category', ['all']))
+                if isinstance(categories, str):
+                    categories = [categories]
                 table = spec.get("table", name)
                 cond = spec.get("cond", "")
                 retries = spec.get("retries", 1)
@@ -448,7 +454,7 @@ class InMemoryQueryExecutorBase(Compute):
                     f"Executing {spec['name']}",
                     extra={
                         "transform": self.name,
-                        "data": json.dumps(spec, indent=4, cls=CustomEncoder),
+                        "data": json.dumps(spec, indent=4, cls=SafeEncoder),
                     },
                 )
 
@@ -577,12 +583,20 @@ class InMemoryQueryExecutorBase(Compute):
                         # Handle a default store for all specs, segments
                         try:
 
+
+                            # => Anonymize the data
+                            if hasattr(self, 'anonargs'):
+                                anon_df1 = self.anonymize_target(spec['name'], df=df1)
+                            else:
+                                anon_df1 = None
+
                             # Store in s3 etc.
                             store_msg, store_dependencies, store_files  = self.store(
-                                segmentcol, segment, df1, spec
+                                segmentcol, segment, df1, anon_df1, spec
                             )
                             if len(store_msg) > 0:
                                 msg += f"[{segment}] " + store_msg
+
                             extra_dependencies += store_dependencies
                             files += store_files
 
@@ -591,9 +605,10 @@ class InMemoryQueryExecutorBase(Compute):
                                 name, engine, sql, df1, extra_dependencies
                             )
 
+
                         except Exception as e:
                             #traceback.print_exc()
-                            msg += "[{}] Exception {}\n".format(segment, str(e))
+                            msg += "[{}] Exception {}\n".format(segment, traceback.format_exc()) + "\n"
 
                     logger.debug(
                         f"Completed {name} {params['start_date']}",
@@ -618,6 +633,7 @@ class InMemoryQueryExecutorBase(Compute):
 
         self.add_marker(state)
 
+
         # Done
         logger.debug(
             "Complete execution", extra=self.config.get_extra({"transform": self.name})
