@@ -466,6 +466,10 @@ class SyntheticDataGeneratorBase(Compute):
                     "get_dataset_s3 expects get_dataset method"
                 )
             datasetobj = self.get_dataset(dataset_main) # this method should be defined in the derived class
+
+            if hasattr(self, 'update_doodle'):
+                self.update_doodle(datasetobj, filename)
+
             df, metadata = datasetobj.read_data(
                 run_date,
                 run_date,
@@ -636,7 +640,7 @@ class SyntheticDataGeneratorBase(Compute):
         self.fakeObj = self.init_faker_object()
 
         # Get the profile spec
-        is_valid, profile, msg = profilespec.get_profile(self, "security")
+        is_valid, profile, msg = profilespec.get_profile(self, "syntheticdata")
         if is_valid:
             logger.debug(
                 f"Loaded profilespec",
