@@ -7,6 +7,8 @@ import json
 import copy
 import shutil
 import logging
+import io
+import hashlib
 import tempfile
 import traceback
 from importlib import import_module
@@ -46,7 +48,7 @@ class CloudMixin(object):
     Mixin baseclass for S3 and GCSFS.
     """
 
-    def _cloud_get_fshandles(self, fshandle=None, bucket=None, fsonly=False, attr="s3"):
+    def _cloud_get_fshandle(self, fshandle=None, attr="s3"):
         """ """
         if fshandle is None:
             fshandle = getattr(self, attr, None)
@@ -61,30 +63,13 @@ class CloudMixin(object):
             )
             raise Exception("Invalid {} handle".format(attr))
 
-        if fsonly:
-            return fshandle, None
+        return fshandle
 
-        if bucket is None:
-            bucket = getattr(self, "bucket", None)
-        if bucket is None:
-            logger.error(
-                "Invalid bucket",
-                extra=self.config.get_extra(
-                    {
-                        "transform": self.name,
-                    }
-                ),
-            )
-            raise Exception("Invalid/missing bucket")
-
-        return fshandle, bucket
-
-    def _cloud_list_files(self, fshandle, bucket, path, include=None):
+    def _cloud_list_files(self, fshandle, path, include=None):
 
         files = []
         try:
-            fullpath = bucket + "/" + path
-            files = fshandle.glob(fullpath)
+            files = fshandle.glob(path)
         except:
             logger.exception(
                 "Unable to access path",
@@ -92,9 +77,6 @@ class CloudMixin(object):
             )
             raise Exception("Unable to access path")
 
-        # lstrip the bucket name..
-        files = [f[len(bucket) + 1 :] for f in files]
-
         # => Split the files
         if include is None:
             include = lambda x: True
@@ -112,10 +94,9 @@ class CloudMixin(object):
 
         return included_files, excluded_files
 
-    def _cloud_list_directories(self, fshandle, bucket, path, normalizer=None):
+    def _cloud_list_directories(self, fshandle, path, normalizer=None):
         files = []
         try:
-            fullpath = bucket + "/" + path
             files = fshandle.ls(path, detail=True)
         except:
             logger.exception(
@@ -140,53 +121,83 @@ class CloudMixin(object):
 
         return dirs, excluded_files
 
-    def _cloud_open_file(self, fshandle, bucket, path):
+    def _cloud_open_file(self, fshandle, path):
 
         try:
-            path = bucket + "/" + path
             fd = fshandle.open(path, "rb")
         except:
             logger.exception(
                 "Unable to read file",
                 extra=self.config.get_extra(
-                    {"transform": self.name, "data": bucket + "/" + path}
+                    {"transform": self.name, "data": path}
                 ),
             )
             raise Exception("Unable to read blockstore file")
 
         return fd
 
-    def _cloud_read_file(self, fshandle, bucket, path):
+    def _cloud_read_file(self, fshandle, path):
 
         try:
-            path = bucket + "/" + path
             content = fshandle.open(path, "rb").read()
         except:
             logger.exception(
                 "Unable to read file",
                 extra=self.config.get_extra(
-                    {"transform": self.name, "data": bucket + "/" + path}
+                    {"transform": self.name, "data": path}
                 ),
             )
             raise Exception("Unable to read blockstore file")
 
         return content
 
-    def _cloud_write_file(self, fshandle, bucket, path, content):
+    def _cloud_write_file(self, fshandle, path, content):
 
         try:
-            path = bucket + "/" + path
+
+            if isinstance(content, str):
+                content = content.encode('utf-8')
+
+            metadata = {
+                "path": path,
+                "sha256": hashlib.sha256(content).hexdigest(),
+                "size": len(content)
+            }
+
             with fshandle.open(path, "wb") as fd:
                 fd.write(content)
+
         except:
             logger.exception(
                 "Unable to write file",
                 extra=self.config.get_extra(
-                    {"transform": self.name, "data": bucket + "/" + path}
+                    {"transform": self.name, "data": path}
                 ),
             )
             raise Exception("Unable to read blockstore file")
 
+        return metadata
+
+    def _cloud_write_frame(self, fshandle, path, df):
+        """
+        Write a dataframe
+        """
+
+        buf = io.StringIO()
+        df.to_csv(buf, index=False)
+        content = buf.getvalue()
+        del buf
+
+        metadata = self._cloud_write_file(fshandle,
+                                          path=path,
+                                          content=content)
+        metadata.update({
+            'rows': df.shape[0],
+            'columns': df.shape[1],
+        })
+
+        return metadata
+
 
 class S3Mixin(CloudMixin):
     """
@@ -229,7 +240,7 @@ class S3Mixin(CloudMixin):
             if "access_key" not in cred:
                 msg += "\naccess_key missing in credentials"
             elif "secret_key" not in cred:
-                msg += "\secret_key missing in credentials"
+                msg += "\nsecret_key missing in credentials"
             else:
                 fail = False
 
@@ -255,29 +266,23 @@ class S3Mixin(CloudMixin):
             client_kwargs=client_kwargs,
         )
 
-    def _s3_get_fshandles(self, s3, bucket=None, fsonly=False):
+    def _s3_get_fshandle(self, s3):
         """
-        Get S3 and bucket tuple. If fsonly specified, then
-        get s3 handle only.
+        Get S3 handle.
 
         Args:
 
            s3 (object): s3fs instance obtained from get_s3_handle
                         If not specified, 's3' object attribute used
-           bucket (str): S3 bucket name
-                        If not specified, 'bucket' object attribute used
-           fsonly (bool): Filesystem handle only (default=False)
 
         Returns:
 
-          tuple: (fileystem handle, bucket name). Bucket name is None if fs only has been specified.
-
+          fileystem handle
         """
-        return self._cloud_get_fshandles(
-            fshandle=s3, bucket=bucket, fsonly=fsonly, attr="s3"
-        )
 
-    def s3_list_files(self, path, bucket=None, s3=None, include=None):
+        return self._cloud_get_fshandle(fshandle=s3, attr='s3')
+
+    def s3_list_files(self, path, s3=None, include=None):
         """
         List files specified by a glob (path)
 
@@ -285,10 +290,7 @@ class S3Mixin(CloudMixin):
 
            s3 (object): s3fs instance obtained from get_s3_handle
                         If not specified, 's3' object attribute used
-           bucket (str): S3 bucket name
-                        If not specified, 'bucket' object attribute used
            path (str): Glob of files
-                       Note that it should not include the bucket name
            include (method): Function that is called for every file to determine whether to include or not
 
         Returns:
@@ -299,12 +301,12 @@ class S3Mixin(CloudMixin):
 
         """
 
-        s3, bucket = self._s3_get_fshandles(s3=s3, bucket=bucket)
+        s3 = self._s3_get_fshandle(s3=s3)
         return self._cloud_list_files(
-            fshandle=s3, bucket=bucket, path=path, include=include
+            fshandle=s3, path=path, include=include
         )
 
-    def s3_list_directories(self, path, bucket=None, s3=None):
+    def s3_list_directories(self, path, s3=None):
         """
         List files specified by a glob (path)
 
@@ -312,10 +314,7 @@ class S3Mixin(CloudMixin):
 
            s3 (object): s3fs instance obtained from get_s3_handle
                         If not specified, 's3' object attribute used
-           bucket (str): S3 bucket name
-                        If not specified, 'bucket' object attribute used
            path (str): Glob of files
-                       Note that it should not include the bucket name
 
         Returns:
 
@@ -325,10 +324,10 @@ class S3Mixin(CloudMixin):
 
         """
 
-        s3, bucket = self._s3_get_fshandles(s3, bucket)
-        return self._cloud_list_directories(fshandle=s3, bucket=bucket, path=path)
+        s3 = self._s3_get_fshandle(s3)
+        return self._cloud_list_directories(fshandle=s3, path=path)
 
-    def s3_open_file(self, path, bucket=None, s3=None):
+    def s3_open_file(self, path, s3=None):
         """
         Open a S3 file and return a file descriptor
 
@@ -336,8 +335,6 @@ class S3Mixin(CloudMixin):
 
           s3 (object): s3fs instance obtained from get_s3_handle
                        If not specified, 's3' object attribute used
-          bucket (str): S3 bucket name
-                        If not specified, 'bucket' object attribute used
           path (str): Path of the file that must be opened
 
         Returns:
@@ -345,10 +342,10 @@ class S3Mixin(CloudMixin):
           object: File descriptor
 
         """
-        s3, bucket = self._s3_get_fshandles(s3, bucket)
-        return self._cloud_open_file(fshandle=s3, bucket=bucket, path=path)
+        s3 = self._s3_get_fshandle(s3)
+        return self._cloud_open_file(fshandle=s3, path=path)
 
-    def s3_read_file(self, path, bucket=None, s3=None):
+    def s3_read_file(self, path, s3=None):
         """
         Read the contents of a S3 object
 
@@ -356,8 +353,6 @@ class S3Mixin(CloudMixin):
 
           s3 (object): s3fs instance obtained from get_s3_handle
                        If not specified, 's3' object attribute used
-          bucket (str): S3 bucket name
-                       If not specified, 'bucket' object attribute used
           path (str): Path of the file that must be opened
 
         Returns:
@@ -365,10 +360,10 @@ class S3Mixin(CloudMixin):
           str: content of the path
 
         """
-        s3, bucket = self._s3_get_fshandles(s3, bucket)
-        return self._cloud_read_file(fshandle=s3, bucket=bucket, path=path)
+        s3 = self._s3_get_fshandle(s3)
+        return self._cloud_read_file(fshandle=s3, path=path)
 
-    def s3_write_file(self, path, content, bucket=None, s3=None):
+    def s3_write_file(self, path, content, s3=None):
         """
         Write the contents to a S3 object
 
@@ -376,41 +371,58 @@ class S3Mixin(CloudMixin):
 
           s3 (object): s3fs instance obtained from get_s3_handle
                        If not specified, 's3' object attribute used
-          bucket (str): S3 bucket name
-                       If not specified, 'bucket' object attribute used
           path (str): Path of the file that must be opened
           content (bytes): Content of the file
 
         Returns:
 
-          str: content of the path
+          metadata (dict): Dictionary having file metadata
 
         """
 
-        s3, bucket = self._s3_get_fshandles(s3, bucket)
+        s3 = self._s3_get_fshandle(s3)
         return self._cloud_write_file(
-            fshandle=s3, bucket=bucket, path=path, content=content
+            fshandle=s3, path=path, content=content
         )
 
-    def s3_cache_files(self, files, localdir, bucket=None, s3=None):
+    def s3_write_frame(self, path, df, s3=None):
+        """
+        Write the contents to a S3 object
+
+        Args:
+
+          s3 (object): s3fs instance obtained from get_s3_handle
+                       If not specified, 's3' object attribute used
+          path (str): Path of the file that must be opened
+          df (dataframe): Dataframe to write
+
+        Returns:
+
+          metadata (dict): Dictionary having file metadata
+
+        """
+        s3 = self._s3_get_fshandle(s3)
+        return self._cloud_write_frame(
+            fshandle=s3, path=path, df=df
+        )
+
+
+    def s3_cache_files(self, files, localdir, s3=None):
         """
         Cache files from S3 into local directory
 
         Args:
-          files (list): List of paths within the bucket
+          files (list): List of paths
           localdir (str): Path to cache the file
           s3 (object): s3fs instance obtained from get_s3_handle
                        If not specified, 's3' object attribute used
-          bucket (str): S3 bucket name
-                        If not specified, 'bucket' object attribute used
         Returns:
           list: list of updated files
         """
 
         assert hasattr(self, "get_cache_dir")
 
-        s3, bucket = self._s3_get_fshandles(s3, bucket)
-
+        s3 = self._s3_get_fshandle(s3)
         metadatafile = os.path.join(localdir, "__cache_metadata__.json")
         try:
             metadata = json.load(open(metadatafile))
@@ -421,12 +433,11 @@ class S3Mixin(CloudMixin):
 
         for f in files:
 
-            fullpath = os.path.join(bucket, f)
             basename = os.path.basename(f)
             localfile = os.path.join(localdir, basename)
 
             # First collect metadata. f doesnt have bucket name
-            info = s3.info(fullpath)
+            info = s3.info(f)
 
             # Fix the format
             info["LastModified"] = info["LastModified"].isoformat()
@@ -548,40 +559,33 @@ class GCSMixin(CloudMixin):
 
         return gcsfs.GCSFileSystem(token=cred["keyfile"])
 
-    def _gcs_get_fshandles(self, gcs, bucket, fsonly=False):
+    def _gcs_get_fshandle(self, gcs):
         """
-        Get GCS handle and bucket tuple. If fsonly specified, then
-        get GCS handle only.
+        Get GCS handle.
 
         Args:
 
            gcs (object): GCSFS instance obtained from get_gcs_handle
                         If not specified, 'gcs' object attribute used
-           bucket (str): GCS bucket name
-                        If not specified, 'bucket' object attribute used
-           fsonly (bool): Filesystem handle only (default=False)
 
         Returns:
 
-          tuple: (fileystem handle, bucket name). Bucket name is None if fs only has been specified.
+          fileystem handle
 
         """
-        return self._cloud_get_fshandles(
-            fshandle=gcs, bucket=bucket, fsonly=fsonly, attr="gcs"
+        return self._cloud_get_fshandle(
+            fshandle=gcs, attr="gcs"
         )
 
-    def gcs_list_files(self, path, gcs=None, bucket=None, include=None):
+    def gcs_list_files(self, path, gcs=None, include=None):
         """
         List files specified by a glob (path)
 
         Args:
 
            path (str): Glob of files
-                       Note that it should not include the bucket name
            gcs (object): gcsfs instance obtained from get_gcs_handle
                         If not specified, 'gcs' object attribute used
-           bucket (str): GCS bucket name
-                        If not specified, 'bucket' object attribute used
            include (def): Function that is called for every file to determine whether to include or not
 
         Returns:
@@ -591,23 +595,20 @@ class GCSMixin(CloudMixin):
            first value is the list of included and the second is the list of excluded
 
         """
-        gcs, bucket = self._gcs_get_fshandles(gcs, bucket)
+        gcs = self._gcs_get_fshandle(gcs)
         return self._cloud_list_files(
-            fshandle=gcs, bucket=bucket, path=path, include=include
+            fshandle=gcs, path=path, include=include
         )
 
-    def gcs_list_directories(self, path, gcs=None, bucket=None):
+    def gcs_list_directories(self, path, gcs=None):
         """
         List files specified by a glob (path)
 
         Args:
 
            path (str): Glob of files
-                       Note that it should not include the bucket name
            gcs (object): gcsfs instance obtained from get_gcs_handle
                         If not specified, 'gcs' object attribute used
-           bucket (str): GCS bucket name
-                        If not specified, 'bucket' object attribute used
 
         Returns:
 
@@ -617,12 +618,12 @@ class GCSMixin(CloudMixin):
 
         """
 
-        gcs, bucket = self._gcs_get_handles(gcs, bucket)
+        gcs = self._gcs_get_handle(gcs)
         return self._cloud_list_directories(
-            fshandle=gcs, bucket=bucket, path=path, normalizer=self.metadata_normalizer
+            fshandle=gcs, path=path, normalizer=self.metadata_normalizer
         )
 
-    def gcs_open_file(self, path, gcs=None, bucket=None):
+    def gcs_open_file(self, path, gcs=None):
         """
         Open a GCS file and return a file descriptor
 
@@ -631,18 +632,16 @@ class GCSMixin(CloudMixin):
           path (str): Path of the file that must be opened
           gcs (object): gcsfs instance obtained from get_gcs_handle
                        If not specified, 'gcs' object attribute used
-          bucket (str): GCS bucket name
-                        If not specified, 'bucket' object attribute used
 
         Returns:
 
           object: File descriptor
 
         """
-        gcs, bucket = self._gcs_get_handles(gcs, bucket)
-        return self._cloud_open_file(fshandle=gcs, bucket=bucket, path=path)
+        gcs = self._gcs_get_handle(gcs)
+        return self._cloud_open_file(fshandle=gcs, path=path)
 
-    def gcs_read_file(self, path, gcs=None, bucket=None):
+    def gcs_read_file(self, path, gcs=None, ):
         """
         Read the contents of a GCS object
 
@@ -651,19 +650,16 @@ class GCSMixin(CloudMixin):
           path (str): Path of the file that must be opened
           gcs (object): gcsfs instance obtained from get_gcs_handle
                        If not specified, 'gcs' object attribute used
-          bucket (str): GCS bucket name
-                       If not specified, 'bucket' object attribute used
-
 
         Returns:
 
           str: content of the path
 
         """
-        gcs, bucket = self._gcs_get_fshandles(gcs, bucket)
-        return self._cloud_read_file(fshandle=gcs, bucket=bucket, path=path)
+        gcs = self._gcs_get_fshandle(gcs)
+        return self._cloud_read_file(fshandle=gcs, path=path)
 
-    def gcs_write_file(self, path, content, gcs=None, bucket=None):
+    def gcs_write_file(self, path, content, gcs=None):
         """
         Write the contents to a GCS object
 
@@ -673,18 +669,38 @@ class GCSMixin(CloudMixin):
           content (bytes): Content of the file
           gcs (object): gcsfs instance obtained from get_gcs_handle
                        If not specified, 'gcs' object attribute used
-          bucket (str): GCS bucket name
-                       If not specified, 'bucket' object attribute used
 
         Returns:
 
-          str: content of the path
+           metadata (dict): Metadata of the output file
 
         """
 
-        gcs, bucket = self._gcs_get_handles(gcs, bucket)
+        gcs = self._gcs_get_handle(gcs, fsonly=True)
         return self._cloud_write_file(
-            fshandle=gcs, bucket=bucket, path=path, content=content
+            fshandle=gcs, path=path, content=content
+        )
+
+    def gcs_write_frame(self, path, df, gcs=None):
+        """
+        Write the contents to a GCS object
+
+        Args:
+
+          path (str): Path of the file that must be opened
+          df (dataframe): Pandas dataframe
+          gcs (object): gcsfs instance obtained from get_gcs_handle
+                       If not specified, 'gcs' object attribute used
+
+        Returns:
+
+           metadata (dict): Metadata of the output file
+
+        """
+
+        gcs = self._gcs_get_handle(gcs)
+        return self._cloud_write_frame(
+            fshandle=gcs, path=path, df=df
         )
 
 
@@ -801,7 +817,7 @@ class PandasMixin(object):
 
         Args:
            name (str): Label for the object. We use it to get the params
-           path (str): Path of the s3 file that must be read including the bucket
+           path (str): Path of the s3 file that must be read including the bucket name
            params (dict): If explicitly passed then the function will not lookup
                           the pandas_params dictionary
            test (bool): Whether this is a test run. This gets a limited number of rows
@@ -1276,14 +1292,17 @@ class EmailMixin(object):
     """
 
     def send_email_helper(
-        self,
-        sender,
-        receivers,
-        subject,
-        body,
-        attachment_prefix="",
-        attachment_suffix="",
-        frames={},
+            self,
+            sender,
+            receivers,
+            subject,
+            body,
+            attachment_prefix="",
+            attachment_suffix="",
+            frames={},
+            reply_to=[],
+            bcc=[],
+            cred={}
     ):
         """
         Send email to a destination..
@@ -1305,9 +1324,12 @@ class EmailMixin(object):
             send_html_email(
                 body,
                 sender=sender,
+                reply_to=reply_to,
+                bcc=bcc,
                 receivers=receivers,
                 subject=subject,
                 attachments=filenames,
+                cred=cred
             )
 
             logger.debug(
@@ -1395,7 +1417,7 @@ class DoodleMixin(object):
                 return
 
             # => First check for doodle spec
-            if 'doodle' not in self.args:
+            if (('doodle' not in self.args) or (isinstance(self.args['doodle'], str))):
                 credname = self.args.get('doodle', 'doodle')
                 self.args['doodle'] = self.get_credentials(credname)
             doodlecred = self.args['doodle']
