@@ -17,6 +17,7 @@ from datetime import datetime
 import abc
 import jsonschema
 
+from ..lib import get_credentials_by_name as lib_get_credentials_by_name
 from ..lib.exceptions import *
 from ..lib import renderlib
 from ..lib.misc import get_checksum, get_file_size
@@ -1204,7 +1205,7 @@ class Node(metaclass=NodeMeta):
     ########################################
     # Helper functions
     ########################################
-    def get_credentials(self, name):
+    def get_credentials_by_name(self, name):
         """
         Looks up the siteconf in ENRICH_ETC and
         returns appropriate entry
@@ -1214,24 +1215,20 @@ class Node(metaclass=NodeMeta):
 
 
         """
-        if not hasattr(self.config, "siteconf"):
-            raise Exception("No siteconf")
-
-        siteconf = self.config.siteconf
-        if not isinstance(siteconf, dict):
-            raise Exception("Invalid siteconf format")
-
         try:
-            credentials = siteconf["credentials"][name]
+            credentials = lib_get_credentials_by_name(name)
         except:
             logger.exception(
-                "Cannot acquire credentials",
+                f"Cannot acquire credentials {name}",
                 extra=self.config.get_extra({"transform": self.name}),
             )
             raise Exception("Missing/invalid credentials")
 
         return credentials
 
+    def get_credentials(self, name):
+        return self.get_credentials_by_name(name)
+
     def get_versionmap(self, include_tags=False):
         """
         Export the versionmap
