@@ -204,7 +204,9 @@ class RunBase(object):
                 cleaned_objects = {}
                 if isinstance(objects, list):
                     # A list of files/text items
-                    for o in objects:
+                    for o in objects[:10]: # limit the number of objects...
+                        if not isinstance(o, str):
+                            continue
                         label = os.path.basename(o) if "/" in o else o
                         cleaned_objects[label] = {
                             "transform": transform,
