@@ -56,6 +56,7 @@ class EnrichStateBase(object):
         self.performance_summary = []
         self.dataset_accesses = []
         self.frames = {}
+        self.messages = {}
         self.transforms = {}
         self.transform_overrides = {}
 
@@ -166,3 +167,39 @@ class EnrichStateBase(object):
             'description': dataset.description,
             'metadata': metadata
         })
+
+    def msgpush(self, from_transform, to_transform, data):
+        """
+        Update the state with the details of a dataframe
+
+        Args:
+            from_transform (str): Name of sender transform
+            to_transform (str): Name of receiver transform
+            message (dict): message
+        """
+        if to_transform not in self.messages:
+            self.messages[to_transform] = []
+
+        # Insert at the end.
+        self.messages[to_transform].append({
+            'from': from_transform,
+            'timestamp': datetime.now().replace(microsecond=0).isoformat(),
+            'data': data
+        })
+
+    def msgpop(self, to_transform):
+        """
+        Update the state with the details of a dataframe
+
+        Args:
+            to_transform (str): Name of receiver transform
+
+        Returns:
+            dict: If there is a message, return it else None
+        """
+
+        messages = self.messages.get(to_transform, [])
+        if len(messages) > 0:
+            return messages.pop()
+
+        return None
