@@ -337,6 +337,32 @@ class DataSource(object):
 
         return spec
 
+    def get_doodle_source(self, filename):
+        """
+        Name as it may be in doodle
+        """
+        params = {
+            'filename': filename
+        }
+
+        if not self.has_subsets():
+            return {
+                'name': self.name,
+                'version': self.version,
+            }
+
+        for s in self.subsets:
+            try:
+                if self.in_subset(s['name'], params):
+                    subsetname = self.resolve_subset_name(s['name'], params)
+                    return {
+                        "name": f"{self.name}-{subsetname}",
+                        "version": self.version
+                    }
+            except:
+                continue
+        raise Exception("Could not construct doodle source name")
+
     def get_path(self, name, full=False, resolve=None):
         return self.get_path_by_name(name, full, resolve)
 
@@ -909,6 +935,7 @@ class DatabaseTable(DataSource):
     def __init__(self, params, *args, **kwargs):
         super().__init__(params, *args, **kwargs)
         self._type = "DBTable"
+        self.match = params.get("match", {})
 
     @property
     def valid_natures(self):
