@@ -492,6 +492,7 @@ class Doodle(object):
     def get_source_paths(self,
                          start, end,
                          name=None,
+                         version="v1",
                          source_id=None):
         """
         Find all the source paths
@@ -500,13 +501,17 @@ class Doodle(object):
             raise Exception("name or source_id must be specified")
 
         if name is not None:
-            sources = self.search_sources(name=name)
-            if len(sources) == 0:
-                raise Exception(f"Source: {name} for found")
-            source = sources[0]
+            sources = self.search_sources(name=name, version=version)
+            if len(sources) > 0:
+                source = sources[0]
+            else:
+                source = self.get_source(source_id)
         else:
             source = self.get_source(source_id)
 
+        if not source['active']:
+            logger.error("Inactive source: {source['id']}")
+
         paths = self.compute_source_paths(source, start, end)
 
         return source, paths
@@ -574,7 +579,8 @@ if __name__ == "__main__":
         try:
             source, paths = doodle.get_source_paths(start=datetime.today() + timedelta(days=-7),
                                                     end=datetime.today(),
-                                                    name=source['name'])
+                                                    name=source['name'],
+                                                    version=source['version'])
 
             print(json.dumps(paths, indent=4))
         except Exception as e:
