@@ -1,8 +1,8 @@
-"""Modules to support feature engineering of objects (dictionaries). 
+"""Modules to support feature engineering of objects (dictionaries).
 
 There are two base classes at the individual feature level and at a
 feature set level. There is a compute function that iterates through
-these for all the objects 
+these for all the objects
 """
 
 import os
@@ -380,9 +380,11 @@ def compute_features(objects, extractors, read_object=None):
 
     # Now collect all features of all patient
     counts = defaultdict(int)
+    times = defaultdict(dict)
     computed = {}
     for detail in extractors:
 
+        t0 = datetime.now()
         name = detail["name"]
         extractor = detail["extractor"]
 
@@ -390,18 +392,32 @@ def compute_features(objects, extractors, read_object=None):
             logger.warning(f"Features missing: {name}", extra={})
             continue
 
+        data = featuresets[name]
+        msg = f"Data: {type(data)} {len(data)}\n"
+
         # Collect all the features into a dataframe..
-        df = extractor.collate(featuresets[name])
+        df = extractor.collate(data)
+        t1 = datetime.now()
 
         # Clean the dataframe generated.
         df = extractor.clean(df)
+        t2 = datetime.now()
 
         computed[name] = df
         counts[name] = df.shape[0]
 
+        msg += f"Shape: {df.shape}\n"
+        msg += f"Collation time  {round((t1-t0).total_seconds(),1)}\n"
+        msg += f"Cleaning time  {round((t2-t1).total_seconds(),1)}\n"
+
+        logger.debug(f"Completed collating {name}",
+                     extra={"data": msg}
+        )
+
+    msg = "records: " + json.dumps(counts, indent=4) + "\n"
     logger.debug(
-        "Completed collating",
-        extra={"data": "records: " + json.dumps(counts, indent=4)},
+        "Completed collating all",
+        extra={"data": msg}
     )
 
     # Now we have individual dataframes. May be the extractor
