@@ -520,7 +520,7 @@ def update_from_git(detail, context, debug=False):
 
         # Extract the tag for this commit
         for ref in refs:
-            match = re.search("tag: refs/tags/(\S+)", ref)
+            match = re.search(r"tag: refs/tags/(\S+)", ref)
             if match is not None:
                 tag = match.group(1)
                 entry["tag"] = tag
