@@ -1,7 +1,8 @@
 import os
 import subprocess
 import uuid
-from hashlib import sha256, md5
+import hashlib
+
 
 
 def get_checksum(path):
@@ -9,11 +10,11 @@ def get_checksum(path):
     if not os.path.isfile(path) or not os.path.exists(path):
         return None
 
-    checksum = subprocess.check_output("/usr/bin/sha256sum {}".format(path), shell=True)
-    #'ffd4d29ee4019150cc308e46004d2f17105899983e7fb4ca7c27ffe981fb82d4  deploy.json\n'
-    checksum = checksum.decode("utf-8").strip()
-    checksum = checksum.split(" ")[0]
-    return checksum
+    sha256_hash = hashlib.sha256()
+    with open(path,"rb") as f:
+        for byte_block in iter(lambda: f.read(4096),b""):
+            sha256_hash.update(byte_block)
+    return sha256_hash.hexdigest()
 
 
 def get_file_size(path):
@@ -29,7 +30,7 @@ def get_consistent_uuid(text):
     # https://stackoverflow.com/questions/41186818/how-to-generate-a-random-uuid-which-is-reproducible-with-a-seed-in-python
 
     for n in range(0, 10):
-        m = md5()
+        m = hashlib.md5()
         m.update((text + " " * n).encode("utf-8"))
         new_uuid = uuid.UUID(m.hexdigest())
         if str(new_uuid)[0] == "0":
