@@ -533,27 +533,27 @@ class MockConfig(object):
             if isinstance(source, str):
                 if "inputdir" not in source:
                     source = "%(inputdir)s/%(transform)s/" + source
-                sources.append(source)
                 path = self.get_file(
                     source, extra={"inputdir": self.inputdir, "transform": transform}
                 )
+                sources.append(path)
                 paths.extend(glob.glob(path))
             elif isinstance(source, list):
                 for f in source:
                     if "inputdir" not in f:
                         f = "%(inputdir)s/%(transform)s/" + f
-                    sources.append(f)
                     path = self.get_file(
                         f, extra={"inputdir": self.inputdir, "transform": transform}
                     )
+                    sources.append(path)
                     paths.extend(glob.glob(path))
             elif isinstance(source, dict):
                 for group in source:
                     for f in source[group]:
-                        sources.append(f)
                         path = self.get_file(
                             f, extra={"transform": transform, "group": group}
                         )
+                        sources.append(path)
                         paths.extend(glob.glob(path))
 
             if len(paths) == 0:
